@@ -4,23 +4,39 @@
 
   Description : 1D/2D/3D Image converter and visualizer based on command line arguments
 
-  Author      : David Tschumperl
+  Copyright  : David Tschumperle - http://www.greyc.ensicaen.fr/~dtschump/
+    
+  This software is governed by the CeCILL  license under French law and
+  abiding by the rules of distribution of free software.  You can  use, 
+  modify and/ or redistribute the software under the terms of the CeCILL
+  license as circulated by CEA, CNRS and INRIA at the following URL
+  "http://www.cecill.info". 
   
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-		
+  As a counterpart to the access to the source code and  rights to copy,
+  modify and redistribute granted by the license, users are provided only
+  with a limited warranty  and the software's author,  the holder of the
+  economic rights,  and the successive licensors  have only  limited
+  liability. 
+  
+  In this respect, the user's attention is drawn to the risks associated
+  with loading,  using,  modifying and/or developing or reproducing the
+  software by the user in light of its specific status of free software,
+  that may mean  that it is complicated to manipulate,  and  that  also
+  therefore means  that it is reserved for developers  and  experienced
+  professionals having in-depth computer knowledge. Users are therefore
+  encouraged to load and test the software's suitability as regards their
+  requirements in conditions enabling the security of their systems and/or 
+  data to be ensured and,  more generally, to use and operate it in the 
+  same conditions as regards security. 
+  
+  The fact that you are presently reading this means that you have had
+  knowledge of the CeCILL license and that you accept its terms.
+   		
   -----------------------------------------------------------------------------------*/
 
 #include "../CImg.h"
 using namespace cimg_library;
-// The lines below should be not necessary in your own code, it simply allows 
-// the source compilation with compilers that do not respect the C++ standart.
+// The lines below are necessary with compilers that do not respect the C++ standart.
 #if ( defined(_MSC_VER) && _MSC_VER<=1200 ) || defined(__DMC__)
 #define std
 #endif
@@ -47,18 +63,21 @@ template<typename T,typename t> void output_file(int argc,const char **argv,CImg
   const char *a_bcrop   = cimg_option("-beg",(const char*)NULL,"Upper-left coordinates crop (2D,3D or 4D coordinates)");
   const char *a_ecrop   = cimg_option("-end",(const char*)NULL,"Lower-right coordinates crop (2D,3D or 4D coordinates");
   const char *a_resize  = cimg_option("-g",(const char*)NULL,"Resize image(s) (ex : '100x100' or '50%x30%')");
+  const int a_interp    = cimg_option("-interp",3,"Resizing interpolation type (0=none, 1=bloc, 2=mosaic, 3=linear, 4=grid, 5=cubic");
   const bool a_equalize = cimg_option("-equalize",false,"Equalize histogram");
   const float a_mul     = (float)cimg_option("-mul",1.0f,"Multiply pixel values by a float constant");
   const float a_add     = (float)cimg_option("-add",0.0f,"Add a float constant to pixel values");
   const int a_erode     = cimg_option("-erode",0,"Erode image(s) (number of steps)");
   const int a_dilate    = cimg_option("-dilate",0,"Dilate image(s) (number of steps)");
   const int a_histo     = cimg_option("-histogram",0,"Compute image histogram (number of levels)");
-  const char a_break    = cimg_option("-b",'\0',"Break image along axis (can be 'z' or 'v')");
+  const char *a_break   = cimg_option("-b",(const char*)NULL,"Break image along axis (can be 'z', 'x3', 'y2', etc...)");
   const char a_append   = cimg_option("-a",'\0',"Append images along axis (can be 'x','y','z' or 'v')");
   const bool a_ucrop    = cimg_option("-crop",false,"User-selected crop");
+  const char *a_annot   = cimg_option("-annotate",(char*)NULL,"Add annotation to image(s)");
+  const bool a_reverse  = cimg_option("-r",false,"Reverse image list");
   const char *file_o    = cimg_option("-o",(const char*)NULL,"Output file");
   const bool a_visu     = cimg_option("-visu",false,"Image visualization");
-  if (!src.size) exit(0);
+  if (!src.size) std::exit(0);
 
   // Display image statistics
   std::fprintf(stderr,"\n- Input image(s) statistics :\n");
@@ -147,14 +166,14 @@ template<typename T,typename t> void output_file(int argc,const char **argv,CImg
       char buf[16];
       if (a_bcrop) std::sscanf(a_bcrop,"%d%15[^0-9]%d%15[^0-9]%d%15[^0-9]%d%15[^0-9]",&x0,buf+0,&y0,buf+1,&z0,buf+2,&v0,buf+3);
       if (a_ecrop) std::sscanf(a_ecrop,"%d%15[^0-9]%d%15[^0-9]%d%15[^0-9]%d%15[^0-9]",&x1,buf+4,&y1,buf+5,&z1,buf+6,&v1,buf+7);
-      if (buf[0]=='%') x0 = -x0*img.width/100;
-      if (buf[1]=='%') y0 = -y0*img.height/100;
-      if (buf[2]=='%') z0 = -z0*img.depth/100;
-      if (buf[3]=='%') v0 = -v0*img.dim/100;
-      if (buf[4]=='%') x1 = -x1*img.width/100;
-      if (buf[5]=='%') y1 = -y1*img.height/100;
-      if (buf[6]=='%') z1 = -z1*img.depth/100;
-      if (buf[7]=='%') v1 = -v1*img.dim/100;
+      if (buf[0]=='%') x0 = -x0*img.dimx()/100;
+      if (buf[1]=='%') y0 = -y0*img.dimy()/100;
+      if (buf[2]=='%') z0 = -z0*img.dimz()/100;
+      if (buf[3]=='%') v0 = -v0*img.dimv()/100;
+      if (buf[4]=='%') x1 = -x1*img.dimx()/100;
+      if (buf[5]=='%') y1 = -y1*img.dimy()/100;
+      if (buf[6]=='%') z1 = -z1*img.dimz()/100;
+      if (buf[7]=='%') v1 = -v1*img.dimv()/100;
       if (x0>x1) cimg::swap(x0,x1);
       if (y0>y1) cimg::swap(y0,y1);
       if (z0>z1) cimg::swap(z0,z1);
@@ -181,7 +200,7 @@ template<typename T,typename t> void output_file(int argc,const char **argv,CImg
     if (buf[1]=='%') dimy=-dimy;
     if (buf[2]=='%') dimz=-dimz;
     if (buf[3]=='%') dimv=-dimv;
-    cimgl_map(src,l) src[l].resize(dimx,dimy,dimz,dimv,3);
+    cimgl_map(src,l) src[l].resize(dimx,dimy,dimz,dimv,a_interp);
   }
 
   if (a_equalize) {
@@ -220,10 +239,12 @@ template<typename T,typename t> void output_file(int argc,const char **argv,CImg
   }
 
   // Image break
-  if (a_break!='\0') {
-    std::fprintf(stderr,"\n- Break image along axis '%c'\t",a_break); std::fflush(stderr);
+  if (a_break) {
+    int nb = 0;
+    std::sscanf(a_break,"%*[xyzvXYZV]%d",&nb);
+    std::fprintf(stderr,"\n- Break image along axis '%c'\t",a_break[0]); std::fflush(stderr);
     CImgl<T> tmp;
-    cimgl_map(src,l) tmp.insert(src[l].get_split(a_break));
+    cimgl_map(src,l) tmp.insert(src[l].get_split(a_break[0],nb));
     src = tmp;
   }
   
@@ -248,60 +269,75 @@ template<typename T,typename t> void output_file(int argc,const char **argv,CImg
     }
   }
 
+  // Annotate images
+  if (a_annot) {
+    std::fprintf(stderr,"\n- Annotate image with \"%s\".\t",a_annot); std::fflush(stderr);
+    cimgl_map(src,l) {
+      const CImg<T> col(src[l].dimv(),1,1,1,(T)CImgStats(src[l],false).max);     
+      src[l].draw_text(a_annot,5,5,col.ptr());
+    }
+  }
+
+  // Reverse image list
+  if (a_reverse) {
+    std::fprintf(stderr,"\n- Reverse image list"); std::fflush(stderr);
+    src.reverse();
+  }
+
   // Convert image to destination type
   std::fprintf(stderr,"\n- Conversion to '%s' pixels\t",cimg::get_type(pixel_type)); std::fflush(stderr);
   CImgl<t> dest(src);
+  std::fprintf(stderr,"\n- Output image(s) :\n");
+  dest.print("Output");
 
   // Save destination image
   if (file_o) {    
     std::fprintf(stderr,"\n- Save image(s) to '%s'\t",file_o); std::fflush(stderr);
     dest.save(file_o);
-    dest.print("\tOutput image(s)");
   }  
 
   // Display destination image
   if (a_visu || !file_o) {
     std::fprintf(stderr,"\n- Visualize image(s)\n"); std::fflush(stderr);
-    if ( dest.size==1 &&
-	 ( dest[0].height==1 && dest[0].depth==1  ) ||
-	 ( dest[0].width==1  && dest[0].depth==1  ) ||
-	 ( dest[0].width==1  && dest[0].height==1 ) ||
-	 ( dest[0].width==1  && dest[0].height==1 && dest[0].depth==1)) { // 1D plot representation
-      const CImg<t>& img = dest[0];
-      img.print("\tOutput image");
+    if ( dest.size==1 && (
+	 ( dest[0].dimy()==1 && dest[0].dimz()==1  ) ||
+	 ( dest[0].dimx()==1  && dest[0].dimz()==1  ) ||
+	 ( dest[0].dimx()==1  && dest[0].dimy()==1 ) ||
+	 ( dest[0].dimx()==1  && dest[0].dimy()==1 && dest[0].dimz()==1))) { // 1D plot representation
+      const CImg<t>& img = dest[0].dimv()<=8?dest[0]:dest[0].get_resize(-100,-100,-100,8,0);
       CImgStats stats(img,false);
-      const CImg<unsigned char> palette = CImg<unsigned char>(3,img.dim,1,1,0).noise(300,1);
+      const CImg<unsigned char> palette = CImg<unsigned char>(3,img.dimv(),1,1,0).noise(300,1);
       const unsigned char gray[3]={128,128,128},white[3]={255,255,255};
       unsigned long trame = 0x0F0F0F0F;
       char message[1024];
       palette[0]=255; palette[1]=0; palette[2]=0;
-      if (img.dim>1) { palette[3]=0; palette[4]=255; palette[5]=0; }
-      if (img.dim>2) { palette[6]=0; palette[7]=0; palette[8]=255; }
+      if (img.dimv()>1) { palette[3]=0; palette[4]=255; palette[5]=0; }
+      if (img.dimv()>2) { palette[6]=0; palette[7]=0; palette[8]=255; }
       CImgDisplay disp(500,300,"1D plot of the output image",0);
       CImg<unsigned char> visu0;
       do {
 	if (!visu0.data || disp.resized) { // plot graphics
 	  disp.resize();
-	  visu0 = CImg<unsigned char>(disp.width,disp.height,1,3,0);
-	  if (img.dim==1) visu0.draw_graph(img,palette.ptr(),1,stats.min,stats.max,0.4f);
-	  cimg_mapV(img,k) visu0.draw_graph(img.ref_channel(k),&palette(0,k),0,stats.min,stats.max);
+	  visu0 = CImg<unsigned char>(disp.dimx(),disp.dimy(),1,3,0);
+	  if (img.dimv()==1) visu0.draw_graph(img,palette.ptr(),1,stats.max,stats.min,0.4f);
+	  cimg_mapV(img,k) visu0.draw_graph(img.ref_channel(k),&palette(0,k),0,stats.max,stats.min);
 	  visu0.draw_axeXY(0,img.size(),stats.max,stats.min,gray);
 	}
 	CImg<unsigned char> visu(visu0);
-	if (disp.mousex>=0) {
-	  visu.draw_line(disp.mousex,0,disp.mousex,visu.height-1,gray,trame,0.5);
+	if (disp.mouse_x>=0) {
+	  visu.draw_line(disp.mouse_x,0,disp.mouse_x,visu.dimy()-1,gray,trame,0.5);
 	  trame = cimg::rol(trame);
-	  const unsigned x = disp.mousex*img.size()/(img.dim*disp.width);
+	  const unsigned x = disp.mouse_x*img.size()/(img.dimv()*disp.dimx());
 	  std::sprintf(message,"x=%d\n[ ",x);
 	  cimg_mapV(img,k) std::sprintf(message+std::strlen(message),"%s%g%s",k==0?"":"  ",
 					(float)img(x,0,0,k),k==img.dimv()-1?"":"\n");
-	  std::sprintf(message+std::strlen(message)," ]");
-	  visu.draw_text(message,disp.mousex-20,10,white,NULL,1);
+	  std::sprintf(message+std::strlen(message)," %s]",dest[0].dimv()<=8?"":"\n...");
+	  visu.draw_text(message,disp.mouse_x-20,10,white,NULL,1);
 	} else visu=visu0;	
 	disp.display(visu).wait(40);	
       } while (!disp.closed && !disp.key);
     }
-    else dest.display("\tOutput image(s)");
+    else dest.display("inrcast");
   }
 }
 
@@ -361,7 +397,7 @@ template<typename T> void input_file(int argc,const char **argv,const T& pixel_t
 //----------------
 int main(int argc,const char **argv) {
   cimg_usage("1D/2D/3D Image(s) converter and visualizer.\n\t\tusage : inrcast [options] image_in1 [image_in2] [...] [-o image_out]");
-  if (argc==1) { std::fprintf(stderr,"Try '%s -h' to get help\n",cimg::basename(argv[0])); exit(0); }
+  if (argc==1) { std::fprintf(stderr,"Try '%s -h' to get help\n",cimg::basename(argv[0])); std::exit(0); }
   
   // Read input image and call converter
   bool casted = false;
@@ -380,6 +416,6 @@ int main(int argc,const char **argv) {
 
   // Return to shell
   std::fprintf(stderr,"Done !\n");
-  exit(0);
+  std::exit(0);
   return 0;
 }
