@@ -4,17 +4,34 @@
 
   Description : Tool for doing presentation from Latex and PS files
 
-  Author      : David Tschumperl
+  Copyright  : David Tschumperle - http://www.greyc.ensicaen.fr/~dtschump/
+  
+  This software is governed by the CeCILL  license under French law and
+  abiding by the rules of distribution of free software.  You can  use, 
+  modify and/ or redistribute the software under the terms of the CeCILL
+  license as circulated by CEA, CNRS and INRIA at the following URL
+  "http://www.cecill.info". 
+  
+  As a counterpart to the access to the source code and  rights to copy,
+  modify and redistribute granted by the license, users are provided only
+  with a limited warranty  and the software's author,  the holder of the
+  economic rights,  and the successive licensors  have only  limited
+  liability. 
+  
+  In this respect, the user's attention is drawn to the risks associated
+  with loading,  using,  modifying and/or developing or reproducing the
+  software by the user in light of its specific status of free software,
+  that may mean  that it is complicated to manipulate,  and  that  also
+  therefore means  that it is reserved for developers  and  experienced
+  professionals having in-depth computer knowledge. Users are therefore
+  encouraged to load and test the software's suitability as regards their
+  requirements in conditions enabling the security of their systems and/or 
+  data to be ensured and,  more generally, to use and operate it in the 
+  same conditions as regards security. 
+  
+  The fact that you are presently reading this means that you have had
+  knowledge of the CeCILL license and that you accept its terms.
   
-  This program is free software; you can redistribute it and/or modify
-  it under the terms of the GNU General Public License as published by
-  the Free Software Foundation; either version 2 of the License, or
-  (at your option) any later version.
-  This program is distributed in the hope that it will be useful,
-  but WITHOUT ANY WARRANTY; without even the implied warranty of
-  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-  GNU General Public License for more details.
-
   ------------------------------------------------------------*/
 
 #include "../CImg.h"
@@ -41,7 +58,7 @@ unsigned int load_Links(const char *Link_filename,Link *Links)
   std::FILE *file = std::fopen(Link_filename,"r");
   unsigned int nol=0,p,x0,y0,w,h;
   if (file) {
-    while (std::fscanf(file,"%d \"%[^\"]\" %d %d %d %d",&p,Links[nol].url,&x0,&y0,&h,&w)==6) {
+    while (std::fscanf(file,"%u \"%[^\"]\" %u %u %u %u",&p,Links[nol].url,&x0,&y0,&h,&w)==6) {
       Links[nol].page=p-1;
       Links[nol].xmin=x0;
       Links[nol].xmax=x0+w;
@@ -75,7 +92,7 @@ void save_Links(Link *Links,const unsigned int nol,const char *Link_filename)
 int main(int argc,char **argv)
 {
   unsigned int ckey=0,stopflag,nop,nol,nom,current_slide,old_slide,update_flag,twidth,theight,slide_type,itmp,i;
-  int mousex,mousey,ooverLink=-1,overLink=-1;
+  int mouse_x,mouse_y,ooverLink=-1,overLink=-1;
   int xmin,xmax,ymin,ymax;
   char slideformat[1024],thumbformat[1024],Link_filename[1024];
   double gamma=1;
@@ -116,32 +133,21 @@ int main(int argc,char **argv)
   int dautorun = cimg_option("-autorun",-1,"If >0, determine a time step (in seconds) that is used for an automatic slideshow");
   int nautolink=-1, autorun=dautorun, nautorun=autorun;
 
-  width = 1024; height = 768; twidth = 128; theight = 96;
-  if (std::sscanf(geometry,"%dx%d",&width,&height)==0) {
-#if cimg_OS==0 || cimg_OS==1
-    {
-      system("xdpyinfo -display :0 | grep dimensions >/tmp/tmp_pslider");
-      file = fopen("/tmp/tmp_pslider","r"); fscanf(file,"%*[^0-9]%dx%d",&width,&height); std::fclose(file);
-    }
-#elif cimg_OS==2
-    {
-      HDC dc = GetDC(NULL);
-      width = GetDeviceCaps(dc,HORZRES);
-      height = GetDeviceCaps(dc,VERTRES);
-    }
-#endif
-  }
-  std::sscanf(tgeometry,"%dx%d",&twidth,&theight);
-  if (cimg_option("-h",false,"Display this help page")) exit(0); else std::fprintf(stderr,"\n*** PSlider ***\n");
+  width = CImgDisplay::screen_dimx(); 
+  height = CImgDisplay::screen_dimy();
+  twidth = 128; theight = 96;
+  std::sscanf(geometry,"%ux%u",&width,&height);
+  std::sscanf(tgeometry,"%ux%u",&twidth,&theight);
+  if (cimg_option("-h",false,"Display this help page")) std::exit(0); else std::fprintf(stderr,"\n*** PSlider ***\n");
   std::fprintf(stderr,"\n - Input geometry : slides=%dx%d, thumbnails=%dx%d\n",width,height,twidth,theight);
 
 // Try to open an existing presentation
   slide_type = 0;
-  std::fprintf(stderr," - Looking for a PS2HTML presentation in current dir ..... "); fflush(stderr);
+  std::fprintf(stderr," - Looking for a PS2HTML presentation in current dir ..... "); std::fflush(stderr);
   if ((file=std::fopen("ps2html_auto/0001.html","r"))!=NULL) { std::fclose(file);
     std::fprintf(stderr, "YES\n"); slide_type = 1; }
   else std::fprintf(stderr,"NO\n");
-  std::fprintf(stderr," - Looking for a PSlider presentation in current dir ..... "); fflush(stderr);
+  std::fprintf(stderr," - Looking for a PSlider presentation in current dir ..... "); std::fflush(stderr);
   if ((file=std::fopen("pslider_auto/.pslider","r"))!=NULL) { std::fclose(file); std::fprintf(stderr,"YES\n"); slide_type = 2; }
   else std::fprintf(stderr,"NO\n");
   if (file_i) {
@@ -339,7 +345,7 @@ int main(int argc,char **argv)
       std::sprintf(Link_filename,"ps2html_auto/links");
       break;
     case 0: // Creating presentation from a PS file
-#if cimg_OS==0 || cimg_OS==1 || cimg_OS==2
+#if cimg_OS==1 || cimg_OS==2
       std::fprintf(stderr," - Creating PSlider presentation from file '%s' :\n",cimg::basename(file_i));
       if (!cimg::strcasecmp("png",format_o)) {
         std::sprintf(slideformat,"pslider_auto/slide_%%d.png");
@@ -352,17 +358,17 @@ int main(int argc,char **argv)
       std::sprintf(Link_filename,"pslider_auto/links");
       ext = cimg::filename_split(file_i);
       stopflag = nop = 0;
-      std::fprintf(stderr,"   > Creating 'PSlider' directory ... "); fflush(stderr);
-#if cimg_OS!=2
-      system("rm -rf pslider_auto");
-      system("mkdir pslider_auto");
+      std::fprintf(stderr,"   > Creating 'PSlider' directory ... "); std::fflush(stderr);
+#if cimg_OS==1
+      std::system("rm -rf pslider_auto");
+      std::system("mkdir pslider_auto");
 #else
-      system("rmdir /s /q pslider_auto");
-      system("mkdir pslider_auto");
+      std::system("rmdir /s /q pslider_auto");
+      std::system("mkdir pslider_auto");
 #endif
       std::fprintf(stderr,"OK\n");
 
-      std::fprintf(stderr,"   > Running Postcript interpreter to generate slides ... "); fflush(stderr);
+      std::fprintf(stderr,"   > Running Postcript interpreter to generate slides ... "); std::fflush(stderr);
       std::FILE *files = cimg::fopen(file_i,"r"), *filed = cimg::fopen("pslider_auto/tmp.ps","w");
       while (std::fscanf(files,"%[^\n]\n",str) && cimg::strcasecmp("%%EndComments",str)) 
         std::fprintf(filed,"%s\n",str);
@@ -477,16 +483,16 @@ int main(int argc,char **argv)
          std::sprintf(str,"%s -q -dBATCH -dNOPAUSE -sPAPERSIZE=a4 -dTextAlphaBits=4 -dGraphicsAlphaBits=4 -r%gx%g -sDEVICE=png16m -sOutputFile=%s pslider_auto/tmp.ps",cimg_OS!=2?"gs":"C:\\gs\\gs8.13\\bin\\gswin32c.exe",resx,resy,slideformat);
       else std::sprintf(str,"%s -q -dBATCH -dNOPAUSE -sPAPERSIZE=a4 -dTextAlphaBits=4 -dGraphicsAlphaBits=4 -r%gx%g -sDEVICE=ppm -sOutputFile=%s pslider_auto/tmp.ps",cimg_OS!=2?"gs":"C:\\gs\\gs8.13\\bin\\gswin32c.exe",resx,resy,slideformat);
 
-      system(str); std::fprintf(stderr,"OK\n");
+      std::system(str); std::fprintf(stderr,"OK\n");
 
       std::fprintf(stderr,"   > Converting images and creating thumbnails :\n");
       while (!stopflag) {
         std::sprintf(str,slideformat,nop+1);
         file = std::fopen(str,"rb");
         if (file) {
-          std::fprintf(stderr,"[%d]",nop+1); fflush(stderr);
+          std::fprintf(stderr,"[%d]",nop+1); std::fflush(stderr);
           slide = CImg<unsigned char>(str);
-          owidth = slide.width; oheight = slide.height;
+          owidth = slide.dimx(); oheight = slide.dimy();
           if (!cimg::strncmp(ext,"ps",3)) { slide.rotate(90); }
           thumb = slide.get_resize(twidth,theight);
           if (orient) { thumb.rotate(180); slide.rotate(180); }
@@ -497,21 +503,21 @@ int main(int argc,char **argv)
           nop++;
         } else stopflag=1;
       }
-#if cimg_OS!=2
-      system("rm -f pslider_auto/tmp.ps pslider_auto/destinations; echo \"\">pslider_auto/.pslider");
+#if cimg_OS==1
+      std::system("rm -f pslider_auto/tmp.ps pslider_auto/destinations; echo \"\">pslider_auto/.pslider");
 #else
-      system("del /q /s pslider_auto\\tmp.ps pslider_auto\\destinations");
-      system("echo \"\">pslider_auto\\.pslider");
+      std::system("del /q /s pslider_auto\\tmp.ps pslider_auto\\destinations");
+      std::system("echo \"\">pslider_auto\\.pslider");
 #endif
       std::fprintf(stderr," OK\n");
       std::fprintf(stderr,"\n** The presentation has been generated, please type 'pslider' again to launch it. **\n\n");
-      exit(0);
+      std::exit(0);
 #endif
       break;
   }
 
 // Check number of pages and original geometry of the presentation
-  std::fprintf(stderr," - Geometry of the original presentation ...."); fflush(stderr);
+  std::fprintf(stderr," - Geometry of the original presentation ...."); std::fflush(stderr);
   stopflag=nop=0;
   while (!stopflag) {
     std::sprintf(str,slideformat,nop+1); file = std::fopen(str,"rb"); if (file!=NULL) {
@@ -521,14 +527,14 @@ int main(int argc,char **argv)
   }
   std::sprintf(str,slideformat,1);
   slide = CImg<unsigned char>(str);
-  owidth = slide.width; oheight = slide.height;
+  owidth = slide.dimx(); oheight = slide.dimy();
   std::fprintf(stderr,"%d slides %dx%d\n",nop,owidth,oheight);
 
 // Load Thumbnails (and optionally slides)
-  std::fprintf(stderr," - Loading Thumbnails ('%s' format)... ",cimg::filename_split(thumbformat)); fflush(stderr);
+  std::fprintf(stderr," - Loading Thumbnails ('%s' format)... ",cimg::filename_split(thumbformat)); std::fflush(stderr);
   {
     for (unsigned int i=0; i<nop; i++) {
-      std::fprintf(stderr,"[%d]",i+1); fflush(stderr);
+      std::fprintf(stderr,"[%d]",i+1); std::fflush(stderr);
       std::sprintf(str,thumbformat,i+1);
       thumb = CImg<unsigned char>(str);
       thumbs.insert(thumb.get_resize(twidth,theight));
@@ -538,14 +544,14 @@ int main(int argc,char **argv)
   std::fprintf(stderr," OK\n");
 
 // Load links
-  std::fprintf(stderr," - Loading Links ... "); fflush(stderr);
+  std::fprintf(stderr," - Loading Links ... "); std::fflush(stderr);
   std::fprintf(stderr,"%d links\n",nol=load_Links(Link_filename,Links));
 
 // Load Mimetypes
   nom = 0;
   if (nol) {
-    std::fprintf(stderr," - Loading mimetypes ... "); fflush(stderr);
-#if cimg_OS!=2
+    std::fprintf(stderr," - Loading mimetypes ... "); std::fflush(stderr);
+#if cimg_OS==1
     std::sprintf(mimes[nom].ext,"mpg");  std::sprintf(mimes[nom++].command,"mtvp -z -l");
     std::sprintf(mimes[nom].ext,"mpeg"); std::sprintf(mimes[nom++].command,"mtvp -z -l");
     std::sprintf(mimes[nom].ext,"avi");  std::sprintf(mimes[nom++].command,"xanim +Sr");
@@ -568,10 +574,10 @@ int main(int argc,char **argv)
 #endif
     file = std::fopen(mimefilename,"r");
     if (file) {               // add user specified mimetypes
-      std::fprintf(stderr,"from file '%s' :",mimefilename); fflush(stderr);
+      std::fprintf(stderr,"from file '%s' :",mimefilename); std::fflush(stderr);
       mime mi;
       while (fscanf(file,"%[^ =]%*[ =]%[ -}] ",mi.ext,mi.command)==2) {
-        std::fprintf(stderr," '%s'",mi.ext); fflush(stderr);
+        std::fprintf(stderr," '%s'",mi.ext); std::fflush(stderr);
         for (itmp=0; itmp<nom; itmp++) if (!cimg::strcasecmp(mi.ext,mimes[itmp].ext)) { strcpy(mimes[itmp].command,mi.command); itmp=nom+1; }
         if (itmp==nom) { strcpy(mimes[nom].command,mi.command); strcpy(mimes[nom++].ext,mi.ext); }
       }
@@ -580,27 +586,30 @@ int main(int argc,char **argv)
     } else std::fprintf(stderr,"no file '%s' found, default used\n",mimefilename);
   }
 
-/*-----------------------------------
-
+  /*-----------------------------------
+    
   Begin User Interaction
-
+  
   ---------------------------------*/
-
+  
   old_slide = nop;
   current_slide = 0;
   update_flag = 1;
-	unsigned int time0 = (unsigned int)time(NULL);
-	bool links_done = false;
- 
+  unsigned int time0 = (unsigned int)time(NULL);
+  bool links_done = false;
+  
   while (!disp || (!disp->closed && ckey!=cimg::keyQ)) {
-    if (!disp) disp = CImg<unsigned char>(width,height).fill(0).new_display("PSlider",0,cimg_OS!=2?2:6);
+    if (!disp) {
+      disp = new CImgDisplay(CImg<unsigned char>(width,height).fill(0),"PSlider",0,2);
+      disp->move(0,0);
+    }
     else { if (!dispthumb && !disp->key && !disp->button && autorun<0) disp->wait(); }
     if (disp->key) { ckey=disp->key; disp->key=0; }
     else ckey=0;
 
 // Go the next slide
     if (ckey==cimg::keyARROWRIGHT || ckey==cimg::keySPACE || ckey==cimg::keyN || ckey==cimg::keyPAGEDOWN || ckey==cimg::keyENTER
-				|| (disp->mousex>=(int)disp->width-8 && disp->button&1) || (autorun>=0 && (unsigned int)time(NULL)>time0+autorun)) {
+				|| (disp->mouse_x>=(int)disp->width-8 && disp->button&1) || (autorun>=0 && (unsigned int)time(NULL)>time0+autorun)) {
       current_slide=(current_slide+1)%nop;
       update_flag=1;
       disp->button&=~1;
@@ -612,7 +621,7 @@ int main(int argc,char **argv)
 			}
     }
 // Go to previous slide
-    if (ckey==cimg::keyARROWLEFT || ckey==cimg::keyB || ckey==cimg::keyPAGEUP || ckey==cimg::keyBACKSPACE || (disp->mousex<=8 && disp->button&1)) {
+    if (ckey==cimg::keyARROWLEFT || ckey==cimg::keyB || ckey==cimg::keyPAGEUP || ckey==cimg::keyBACKSPACE || (disp->mouse_x<=8 && disp->button&1)) {
       current_slide=(current_slide+nop-1)%nop;
       update_flag = 1;
       disp->button&=~1;
@@ -645,7 +654,7 @@ int main(int argc,char **argv)
 		// Add a link
     if (ckey==cimg::keyF1) {
       int bbox[6];
-      std::fprintf(stderr," > Add link #%d : ",nol+1); fflush(stderr);
+      std::fprintf(stderr," > Add link #%d : ",nol+1); std::fflush(stderr);
       vslide.feature_selection(bbox,2,*disp);
       int x0=bbox[0],y0=bbox[1],x1=bbox[3],y1=bbox[4];
       x0 = x0*owidth/width;
@@ -669,7 +678,7 @@ int main(int argc,char **argv)
     }
     // Remove a Link
     if (overLink>=0 && ckey==cimg::keyF2) {
-      std::fprintf(stderr," > Remove link %d...",overLink+1); fflush(stderr);
+      std::fprintf(stderr," > Remove link %d...",overLink+1); std::fflush(stderr);
       for (unsigned int i=overLink; i<nol-1; i++) Links[i]=Links[i+1];
       nol--;
       save_Links(Links,nol,Link_filename);
@@ -678,7 +687,7 @@ int main(int argc,char **argv)
     }
     // Insert a new link page
     if (ckey==cimg::keyF3) {
-      std::fprintf(stderr," > Insert a new link page ... "); fflush(stderr);
+      std::fprintf(stderr," > Insert a new link page ... "); std::fflush(stderr);
       for (unsigned int i=0; i<nol; i++) if (Links[i].page>=current_slide) Links[i].page++;
       save_Links(Links,nol,Link_filename);
       std::fprintf(stderr," '%s' updated\n",Link_filename);
@@ -686,7 +695,7 @@ int main(int argc,char **argv)
     }
     // Remove a link page
     if (ckey==cimg::keyF4) {
-      std::fprintf(stderr," > Remove a link page ..."); fflush(stderr);
+      std::fprintf(stderr," > Remove a link page ..."); std::fflush(stderr);
       for (unsigned int i=0; i<nol; i++) {
         if (Links[i].page==current_slide) { for (unsigned int k=i; k<nol-1; k++) Links[k]=Links[k+1]; nol--; i--; }
         else { if (Links[i].page>current_slide) Links[i].page--; }
@@ -697,11 +706,11 @@ int main(int argc,char **argv)
     }
     // Edit link file
     if (ckey==cimg::keyF5) {
-      if (cimg_OS!=2) {
+      if (cimg_OS==1) {
         std::fprintf(stderr," > Edit link file : ");
         std::sprintf(str,"xemacs %s",Link_filename);
         std::fprintf(stderr,"'%s'\n",str);
-        system(str);
+        std::system(str);
         nol=load_Links(Link_filename,Links);
         update_flag=1;
       }
@@ -739,11 +748,11 @@ int main(int argc,char **argv)
 \t  3        : Lighten the slides\n\
 \t  4        : Reset the gamma correction\n\
         ",20,20,white);
-      help_win.draw_line(4,4,help_win.width-5,4,white);
-      help_win.draw_line(help_win.width-5,4,help_win.width-5,help_win.height-5,white);
-      help_win.draw_line(help_win.width-5,help_win.height-5,4,help_win.height-5,white);
-      help_win.draw_line(4,help_win.height-5,4,4,white);
-      vslide.draw_image(help_win,(vslide.width-help_win.width)/2,(vslide.height-help_win.height)/2,0,0,0.75);
+      help_win.draw_line(4,4,help_win.dimx()-5,4,white);
+      help_win.draw_line(help_win.dimx()-5,4,help_win.dimx()-5,help_win.dimy()-5,white);
+      help_win.draw_line(help_win.dimx()-5,help_win.dimy()-5,4,help_win.dimy()-5,white);
+      help_win.draw_line(4,help_win.dimy()-5,4,4,white);
+      vslide.draw_image(help_win,(vslide.dimx()-help_win.dimx())/2,(vslide.dimy()-help_win.dimy())/2,0,0,0.75);
 
       disp->key=0;
       vslide.display(*disp);
@@ -756,20 +765,20 @@ int main(int argc,char **argv)
 // Thumbnails menu
 //------------------
     if (ckey==cimg::keyESC || disp->button&2 || (dispthumb && (dispthumb->closed || dispthumb->key==cimg::keyESC || dispthumb->button&2))) {
-
+      
       if (!dispthumb) {
-// Open menu
-        unsigned int fx=thumbs[0].width+8,fy=thumbs[0].height+8,x,y;
+	// Open menu
+        unsigned int fx=thumbs[0].dimx()+8,fy=thumbs[0].dimy()+8,x,y;
         menuthumb = CImg<unsigned char>((nop<8?nop*fx:8*fx),fy*(1+(nop-1)/8),1,3).fill(0);
         { cimg_mapXY(menuthumb,x,y) menuthumb(x,y,0)=menuthumb(x,y,1)=menuthumb(x,y,2)=(unsigned char)(std::cos(x/30.0)*std::sin(y/30.0)*64)+64; }
-        dispthumb = menuthumb.new_display("Presentation Layout",0,2);
+        dispthumb = new CImgDisplay(menuthumb,"Presentation Layout",0,2);
         x=y=0;
         for (unsigned int k=0; k<nop; k++) {
           menuthumb.draw_image(thumbs[k],x+4,y+4,0,0);
-          menuthumb.draw_line(x+3,y+3,x+3+thumbs[k].width+1,y+3,white);
-          menuthumb.draw_line(x+3+thumbs[k].width+1,y+3,x+3+thumbs[k].width+1,y+3+thumbs[k].height+1,white);
-          menuthumb.draw_line(x+3+thumbs[k].width+1,y+3+thumbs[k].height+1,x+3,y+3+thumbs[k].height+1,gray);
-          menuthumb.draw_line(x+3,y+3+thumbs[k].height+1,x+3,y+3,gray);
+          menuthumb.draw_line(x+3,y+3,x+3+thumbs[k].dimx()+1,y+3,white);
+          menuthumb.draw_line(x+3+thumbs[k].dimx()+1,y+3,x+3+thumbs[k].dimx()+1,y+3+thumbs[k].dimy()+1,white);
+          menuthumb.draw_line(x+3+thumbs[k].dimx()+1,y+3+thumbs[k].dimy()+1,x+3,y+3+thumbs[k].dimy()+1,gray);
+          menuthumb.draw_line(x+3,y+3+thumbs[k].dimy()+1,x+3,y+3,gray);
           x+=fx;
           if (x>=menuthumb.width) { x=0; y+=fy; }
         }
@@ -777,7 +786,7 @@ int main(int argc,char **argv)
         disp->button&=~2;
       }
       else {
-// Close menu
+	// Close menu
         delete dispthumb;
         dispthumb=NULL;
         disp->button&=~2;
@@ -786,10 +795,10 @@ int main(int argc,char **argv)
     }
 
     if (dispthumb) {
-      int next=current_slide,x,y,fx=thumbs[0].width+8,fy=thumbs[0].height+8,x0,x1,y0,y1;
+      int next=current_slide,x,y,fx=thumbs[0].dimx()+8,fy=thumbs[0].dimy()+8,x0,x1,y0,y1;
       CImg<unsigned char> tmp(menuthumb);
-      x = dispthumb->mousex/fx;
-      y = dispthumb->mousey/fy;
+      x = dispthumb->mouse_x/fx;
+      y = dispthumb->mouse_y/fy;
       if (x>=0 && y>=0 && y*8+x<(int)nop) {
         x0 = x*fx; y0 = y*fy;
         x1 = (x+1)*fx-1; y1 = (y+1)*fy-1;
@@ -856,8 +865,8 @@ int main(int argc,char **argv)
     ooverLink = overLink;
     overLink = -1;
     layer = vslide;
-    mousex = disp->mousex;
-    mousey = disp->mousey;
+    mouse_x = disp->mouse_x;
+    mouse_y = disp->mouse_y;
 		
     if (autorun<0 || (autorun>=0 && (unsigned int)time(NULL)>time0+3*autorun/4)) {
 			for (i=0; i<nol; i++) {
@@ -875,7 +884,7 @@ int main(int argc,char **argv)
                                   std::sscanf(Links[i].url,"file:%d.settimer",&nautorun);
                                   std::fprintf(stderr," > Next slide timer will be %d seconds\n",nautorun);
 				}
-				if (autorun<0 && Links[i].page==current_slide && mousex>=xmin && mousex<=xmax && mousey>=ymin && mousey<=ymax
+				if (autorun<0 && Links[i].page==current_slide && mouse_x>=xmin && mouse_x<=xmax && mouse_y>=ymin && mouse_y<=ymax
 					&& cimg::strncasecmp(Links[i].url,"settimer",8) ) {
 					overLink = i;
 					if (!noLink) {
@@ -918,11 +927,11 @@ int main(int argc,char **argv)
           if (!cimg::strncasecmp(l.url,"http://",7) || !cimg::strncasecmp(l.url,"ftp://",6) ||
             !cimg::strncasecmp(l.url,"www.",4)
             ) std::sprintf(str,"%s %s",mimes[m].command,l.url);
-          if (cimg_OS!=2) {
+          if (cimg_OS==1) {
 						if (autorun<0) std::sprintf(str+cimg::strlen(str)," &");
 						//else std::sprintf(str+cimg::strlen(str),"");
 					}
-          std::fprintf(stderr," > Launching '%s' ... ",str); fflush(stderr); system(str);
+          std::fprintf(stderr," > Launching '%s' ... ",str); std::fflush(stderr); std::system(str);
 					if (autorun>=0) time0 = (unsigned int)time(NULL)-(unsigned int)3*autorun/4;
 					fprintf(stderr,"Done\n");
           m=nom;
@@ -934,7 +943,5 @@ int main(int argc,char **argv)
 
 // Exit pslider.
   std::fprintf(stderr,"\n*** Exit PSlider ***\n\n");
-
-  exit(0);
   return 0;
 }
